import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';

class StartAppSdk {
  static final StartAppSdk _instance = StartAppSdk._();

  final MethodChannel _channel = const MethodChannel('com.startapp.flutter');

  factory StartAppSdk() {
    return _instance;
  }

  StartAppSdk._();

  Future getSdkVersion() {
    return _channel.invokeMethod('getSdkVersion');
  }

  Future setTestAdsEnabled(bool value) {
    return _channel.invokeMethod('setTestAdsEnabled', value);
  }

  Future<StartAppBannerAd> loadBannerAd(
    StartAppBannerType type, [
    StartAppAdPreferences prefs = const StartAppAdPreferences(),
  ]) {
    return _channel.invokeMethod('loadBannerAd', prefs._toMap({'type': type.index})).then((value) {
      if (value is Map) {
        dynamic id = value['id'];

        if (id is int && id > 0) {
          return StartAppBannerAd._(id, value.cast());
        }
      }

      throw StartAppException(message: value);
    });
  }

  Future<StartAppInterstitialAd> loadInterstitialAd([
    StartAppAdPreferences prefs = const StartAppAdPreferences(),
  ]) {
    return _channel.invokeMethod('loadInterstitialAd', prefs._toMap()).then((value) {
      if (value is Map) {
        dynamic id = value['id'];

        if (id is int && id > 0) {
          return StartAppInterstitialAd._(_channel, id);
        }
      }

      throw StartAppException(message: value);
    });
  }

  Future<StartAppNativeAd> loadNativeAd([
    StartAppAdPreferences prefs = const StartAppAdPreferences(),
  ]) {
    return _channel.invokeMethod('loadNativeAd', prefs._toMap()).then((value) {
      if (value is Map) {
        dynamic id = value['id'];

        if (id is int && id > 0) {
          return StartAppNativeAd._(id, value.cast());
        }
      }

      throw StartAppException(message: value);
    });
  }
}

class StartAppAdPreferences {
  final String? adTag;
  final String? keywords;
  final String? gender;
  final int? age;
  final bool? videoMuted;
  final bool? hardwareAccelerated;
  final List<String>? categories;
  final List<String>? categoriesExclude;

  const StartAppAdPreferences({
    this.adTag,
    this.keywords,
    this.gender,
    this.age,
    this.videoMuted,
    this.hardwareAccelerated,
    this.categories,
    this.categoriesExclude,
  });

  Map<String, dynamic> _toMap([Map<String, dynamic>? map]) {
    if (map == null) {
      map = {};
    }

    map.addAll({
      'adTag': adTag,
      'keywords': keywords,
      'gender': gender,
      'age': age,
      'videoMuted': videoMuted,
      'hardwareAccelerated': hardwareAccelerated,
      'categories': categories,
      'categoriesExclude': categoriesExclude,
    });

    return map;
  }
}

enum StartAppBannerType {
  BANNER,
  MREC,
  COVER,
}

class StartAppBannerAd {
  final int _id;
  final Map<String, dynamic> _data;

  StartAppBannerAd._(this._id, this._data);

  double? get width {
    dynamic value = _data['width'];
    return value is num ? value.toDouble() : null;
  }

  double? get height {
    dynamic value = _data['height'];
    return value is num ? value.toDouble() : null;
  }
}

class StartAppBanner extends StatefulWidget {
  final StartAppBannerAd _ad;

  StartAppBanner(this._ad);

  @override
  State createState() {
    return _StartAppAndroidViewState(
      'com.startapp.flutter.Banner',
      creationParams: {
        'adId': _ad._id,
      },
      width: _ad.width,
      height: _ad.height,
    );
  }
}

class StartAppInterstitialAd {
  final MethodChannel _channel;
  final int _id;

  StartAppInterstitialAd._(this._channel, this._id);

  Future<bool> show() {
    return _channel.invokeMethod('showInterstitialAd', {
      'id': _id,
    }).then((value) {
      return value is bool && value;
    });
  }
}

class StartAppNativeAd {
  final int _id;
  final Map<String, dynamic> _data;

  StartAppNativeAd._(this._id, this._data);

  String? get title {
    return _data['title'];
  }

  String? get description {
    return _data['description'];
  }

  double? get rating {
    return _data['rating'];
  }

  String? get installs {
    return _data['installs'];
  }

  String? get category {
    return _data['category'];
  }

  String? get campaign {
    return _data['campaign'];
  }

  String? get callToAction {
    return _data['callToAction'];
  }

  String? get imageUrl {
    return _data['imageUrl'];
  }

  String? get secondaryImageUrl {
    return _data['secondaryImageUrl'];
  }
}

typedef StartAppNativeWidgetBuilder = Widget Function(BuildContext context, StateSetter setState, StartAppNativeAd nativeAd);

class StartAppNative extends StatefulWidget {
  final StartAppNativeAd _ad;
  final StartAppNativeWidgetBuilder _builder;
  final double? width;
  final double? height;
  final bool ignorePointer;

  StartAppNative(
    this._ad,
    this._builder, {
    this.width,
    this.height,
    this.ignorePointer = true,
  });

  @override
  State<StatefulWidget> createState() {
    return _StartAppAndroidViewState(
      'com.startapp.flutter.Native',
      creationParams: {
        'adId': _ad._id,
        'width': width,
        'height': height,
      },
      width: width,
      height: height,
      overlayBuilder: (context, setState) {
        return IgnorePointer(
          ignoring: ignorePointer,
          child: _builder(context, setState, _ad),
        );
      },
    );
  }
}

class _StartAppAndroidViewState extends State {
  final String viewType;
  final Map<String, dynamic> creationParams;
  final double? width;
  final double? height;
  final StatefulWidgetBuilder? overlayBuilder;

  _StartAppAndroidViewState(
    this.viewType, {
    this.creationParams = const {},
    this.width,
    this.height,
    this.overlayBuilder,
  });

  Widget build(BuildContext context) {
    return SizedBox(
      width: width,
      height: height,
      child: Stack(
        children: [
          PlatformViewLink(
            viewType: viewType,
            surfaceFactory: (BuildContext context, PlatformViewController controller) {
              return AndroidViewSurface(
                controller: controller as AndroidViewController,
                gestureRecognizers: const <Factory<OneSequenceGestureRecognizer>>{},
                hitTestBehavior: PlatformViewHitTestBehavior.opaque,
              );
            },
            onCreatePlatformView: (PlatformViewCreationParams params) {
              return PlatformViewsService.initSurfaceAndroidView(
                id: params.id,
                viewType: viewType,
                layoutDirection: TextDirection.ltr,
                creationParams: creationParams,
                creationParamsCodec: StandardMessageCodec(),
              )
                ..addOnPlatformViewCreatedListener(params.onPlatformViewCreated)
                ..create();
            },
          ),
          overlayBuilder != null ? StatefulBuilder(builder: overlayBuilder!) : Container(),
        ],
      ),
    );
  }
}

class StartAppException implements Exception {
  final dynamic message;

  StartAppException({this.message});
}
